################################################################################
# Copyright (c) 2019, NVIDIA CORPORATION. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
################################################################################

#!/usr/bin/env python3

import sys
import numpy
sys.path.append('../')
import gi
gi.require_version('Gst', '1.0')
from gi.repository import GObject, Gst
from common.is_aarch_64 import is_aarch64
from common.bus_call import bus_call
import cv2

import pyds

PGIE_CLASS_ID_BOLA = 0


def main(args):
    # Check input arguments
    if len(args) != 2:
        sys.stderr.write("usage: %s <v4l2-device-path>\n" % args[0])
        sys.exit(1)

    # Standard GStreamer initialization
    GObject.threads_init()
    Gst.init(None)

    # Create gstreamer elements
    # Create Pipeline element that will form a connection of other elements
    print("Creating Pipeline \n ")
    pipeline = Gst.Pipeline()

    if not pipeline:
        sys.stderr.write("Unable to create pipeline \n")

    #Source Camera
    print("Creating Source")
    source = Gst.ElementFactory.make("v4l2src","usb-cam-source")
    if not source:
        sys.stderr.write(" Unable to create Source \n")

    source_caps = Gst.ElementFactory.make("capsfilter","source_caps")
    if not source_caps:
        sys.stderr.write(" Unable to create v4l2src capsfilter \n")

    #Video Converter
    videoconvert = Gst.ElementFactory.make("videoconvert","convert_1")
    if not videoconvert:
        sys.stderr.write("Unable to create videoconvert \n")

    nvvideoconvert = Gst.ElementFactory.make("nvvideoconvert","convert_2")
    if not videoconvert:
        sys.stderr.write("Unable to create videoconvert_2 \n")

    #Caps Filter (NVMM)
    converter_caps = Gst.ElementFactory.make("capsfilter","convert_caps")
    if not converter_caps:
        sys.stderr.write("Unable to create converter caps \n")

    #Caps Filter (NVMM format RGBA)
    converter_caps2 = Gst.ElementFactory.make("capsfilter","convert_caps2")
    if not converter_caps:
        sys.stderr.write("Unable to create converter caps NVVM format RGBA\n")

    #Caps Filter (Format RGBA)
    converter_caps3 = Gst.ElementFactory.make("capsfilter","convert_caps3")
    if not converter_caps:
        sys.stderr.write("Unable to create converter caps format RGBA \n")

    #Caps Filter (Format BGR)
    converter_caps4 = Gst.ElementFactory.make("capsfilter","convert_caps4")
    if not converter_caps:
        sys.stderr.write("Unable to create converter caps format BGR \n")

    #Stream Mux
    streammux = Gst.ElementFactory.make("nvstreammux", "Stream-muxer")
    if not streammux:
        sys.stderr.write(" Unable to create NvStreamMux \n")

    #Inference
    pgie = Gst.ElementFactory.make("nvinfer","inference")
    if not pgie:
        sys.stderr.write(" Unable to create pgie \n")

    #Display
    nvosd = Gst.ElementFactory.make("nvdsosd","display")
    if not nvosd:
        sys.stderr.write(" Unable to create nvosd \n")

    print("Playing Camera %s " %args[1])

    #Sink
    sink = Gst.ElementFactory.make("appsink","sink")
    if not sink:
        sys.stderr.write(" Unable to create sink \n")

    source.set_property('device', args[1])
    
    source_caps.set_property('caps', Gst.Caps.from_string("video/x-raw, framerate=30/1"))
              
    converter_caps.set_property('caps', Gst.Caps.from_string("video/x-raw(memory:NVMM)"))

    converter_caps2.set_property('caps', Gst.Caps.from_string("video/x-raw(memory:NVMM), format=RGBA"))

    converter_caps3.set_property('caps', Gst.Caps.from_string("video/x-raw, format=RGBA"))

    converter_caps4.set_property('caps', Gst.Caps.from_string("video/x-raw, format=BGR"))
    
    streammux.set_property('width',640)
    streammux.set_property('height',480)
    streammux.set_property('batch-size',1)
    streammux.set_property('batched-push-timeout',4000000)

    pgie.set_property('config-file-path',"dstest1_pgie_config.txt")
    
    print("Adding elements to Pipeline")
    pipeline.add(source)
    pipeline.add(source_caps)
    pipeline.add(videoconvert)
    pipeline.add(nvvideoconvert)
    pipeline.add(converter_caps)
    pipeline.add(streammux)
    pipeline.add(pgie)
    pipeline.add(nvvideoconvert)
    pipeline.add(nvosd)
    pipeline.add(converter_caps2)
    pipeline.add(nvvideoconvert)
    pipeline.add(converter_caps3)
    pipeline.add(videoconvert)
    pipeline.add(converter_caps4)
    pipeline.add(sink)

    print("Linking elements")
    source.link(source_caps)
    source_caps.link(videoconvert)
    videoconvert.link(nvvideoconvert)
    nvvideoconvert.link(converter_caps)

    sinkpad = streammux.get_request_pad("sink_0")
    if not sinkpad:
        sys.stderr.write(" Unable to get the sink pad of streammux \n")
    srcpad = converter_caps.get_static_pad("src")
    if not srcpad:
        sys.stderr.write(" Unable to get source pad of converter_caps \n")
    srcpad.link(sinkpad)
    streammux.link(pgie)
    pgie.link(nvvideoconvert)
    nvvideoconvert.link(nvosd)
    nvosd.link(converter_caps2)
    converter_caps2.link(nvvideoconvert)
    nvvideoconvert.link(converter_caps3)
    converter_caps3.link(videoconvert)
    videoconvert.link(converter_caps4)
    converter_caps4.link(sink)

    ret = pipeline.set_state(Gst.State.PLAYING)
    if ret == Gst.StateChangeReturn.FAILURE:
        print("Unable to set the pipeline to the playing state.")
        exit(-1)

    bus = pipeline.get_bus()

    print("Starting Pipeline")
    while True:
        cv2.imshow("frame",ret)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    # Free resources
    pipeline.set_state(Gst.State.NULL)

if __name__ == '__main__':
    sys.exit(main(sys.argv))

